'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _findMatchingTextIndex = require('./utils/findMatchingTextIndex');

var _findMatchingTextIndex2 = _interopRequireDefault(_findMatchingTextIndex);

var _getInput = require('./utils/getInput');

var _getInput2 = _interopRequireDefault(_getInput);

var _registerInput = require('./utils/registerInput');

var _registerInput2 = _interopRequireDefault(_registerInput);

var _renderChild = require('./utils/renderChild');

var _renderChild2 = _interopRequireDefault(_renderChild);

var _toString = require('./utils/toString');

var _toString2 = _interopRequireDefault(_toString);

var _getOptionText = require('./utils/getOptionText');

var _getOptionText2 = _interopRequireDefault(_getOptionText);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function updateInputSelection(input, start, end) {
  input.setSelectionRange(start, end);
}


function updateInputNode(input, value) {
  input.value = value;
}

function setSelection(input, text, matchingText) {
  if (text === null) {
    updateInputNode(input, '');
  } else {
    updateInputNode(input, matchingText);

    if (text.length !== matchingText.length) {
      updateInputSelection(input, text.length, matchingText.length);
    }
  }
}

function getStateFromProps(_ref) {
  var value = _ref.value;
  var options = _ref.options;

  var match = (0, _findMatchingTextIndex2.default)(value, options);

  if (!match.noResult) {
    var _matchingText = match.text;

    var strValue = typeof value === 'string' ? value : (0, _getOptionText2.default)(options[match.index]);
    return { value: strValue, matchingText: _matchingText };
  } else {
    return { value: (0, _toString2.default)(value), matchingText: '' };
  }
}

var Autocomplete = function (_PureComponent) {
  (0, _inherits3.default)(Autocomplete, _PureComponent);

  function Autocomplete(props) {
    (0, _classCallCheck3.default)(this, Autocomplete);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Autocomplete.__proto__ || (0, _getPrototypeOf2.default)(Autocomplete)).call(this, props));

    _this.registerInput = function (input) {
      return (0, _registerInput2.default)(_this, input);
    };

    _this.handleChange = function (e) {
      var value = e.target.value;

      var nextState = getStateFromProps((0, _extends3.default)({}, _this.props, { value: value }));

      if (nextState.value !== _this.state.value) {
        _this.setState(nextState);
      }

      if (_this.props.onChange) {
        _this.props.onChange(e);
      }
    };

    _this.handleKeyDown = function (e) {
      var keyMap = {
        Backspace: _this.handleBackspaceKeyDown,
        Enter: _this.handleEnterKeyDown
      };

      if (keyMap[e.key]) {
        keyMap[e.key](e);
      }

      if (_this.props.onKeyDown) {
        _this.props.onKeyDown(e);
      }
    };

    _this.handleBackspaceKeyDown = function () {
      var input = (0, _getInput2.default)(_this);

      if (input.selectionStart !== input.selectionEnd && input.selectionEnd === input.value.length && input.selectionStart !== 0) {
        var _value = input.value.substr(0, input.selectionStart);
        var updatedValue = _value.substr(0, _value.length - 1);

        var nextState = getStateFromProps((0, _extends3.default)({}, _this.props, { value: updatedValue }));
        _this.setState(nextState);

        updateInputNode(input, _value);
      }
    };

    _this.handleEnterKeyDown = function () {
      var matchingText = _this.state.matchingText;

      var input = (0, _getInput2.default)(_this);

      setSelection(input, matchingText, matchingText);
      input.blur();
    };

    _this.state = getStateFromProps(props);
    return _this;
  }

  (0, _createClass3.default)(Autocomplete, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var optionsChanged = this.props.options !== nextProps.options;
      if (this.props.value !== nextProps.value || optionsChanged) {
        var nextState = getStateFromProps(nextProps);
        if (nextState.value !== this.state.value || optionsChanged) {
          this.setState(nextState);
        }
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      var _state = this.state;
      var value = _state.value;
      var matchingText = _state.matchingText;


      if (matchingText && value.length !== matchingText.length) {
        var _input = (0, _getInput2.default)(this);
        setSelection(_input, this.state.value, this.state.matchingText);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var children = this.props.children;
      var _state2 = this.state;
      var matchingText = _state2.matchingText;
      var value = _state2.value;

      var inputProps = {
        value: matchingText || value,
        onKeyDown: this.handleKeyDown,
        onChange: this.handleChange
      };

      return (0, _renderChild2.default)(children, inputProps, { matchingText: matchingText, value: value }, this.registerInput);
    }
  }]);
  return Autocomplete;
}(_react.PureComponent);

exports.default = Autocomplete;