'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

require('./utils/getComputedStyle');

var _getInput = require('./utils/getInput');

var _getInput2 = _interopRequireDefault(_getInput);

var _registerInput = require('./utils/registerInput');

var _registerInput2 = _interopRequireDefault(_registerInput);

var _renderChild = require('./utils/renderChild');

var _renderChild2 = _interopRequireDefault(_renderChild);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ALLOWED_CSS_PROPS = ['direction', 'fontFamily', 'fontKerning', 'fontSize', 'fontSizeAdjust', 'fontStyle', 'fontWeight', 'letterSpacing', 'lineHeight', 'padding', 'textAlign', 'textDecoration', 'textTransform', 'wordSpacing'];


var toDashCase = function toDashCase(str) {
  return str.replace(/([A-Z])/g, function (m, l) {
    return '-' + l.toLowerCase();
  });
};

var sizersListEl = null;

var sizerContainerStyle = {
  position: 'absolute',
  visibility: 'hidden',
  whiteSpace: 'pre',
  width: 'auto',
  minWidth: 'initial',
  maxWidth: 'initial',
  zIndex: 10000,
  left: -1000,
  top: 100
};

function getStateFromProps(props, stateValue, initialWidth, input, sizerEl) {
  var widthValue = (props.value === undefined ? stateValue : props.value) || props.placeholder;
  var inputStyle = input && window.getComputedStyle(input, null);

  var width = void 0;

  if (!widthValue || !sizerEl || !input) {
    width = props.defaultWidth || initialWidth || 0;
  } else {
    for (var key in inputStyle) {
      if (ALLOWED_CSS_PROPS.indexOf(key) !== -1) {
        sizerEl.style.setProperty(toDashCase(key), inputStyle[key]);
      }
    }

    sizerEl.innerText = widthValue;

    width = Math.max(sizerEl.offsetWidth + props.padding + 1, props.defaultWidth || initialWidth || 0);
  }

  return {
    width: width,
    value: stateValue
  };
}

var Autosize = function (_PureComponent) {
  (0, _inherits3.default)(Autosize, _PureComponent);

  function Autosize(props) {
    (0, _classCallCheck3.default)(this, Autosize);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Autosize.__proto__ || (0, _getPrototypeOf2.default)(Autosize)).call(this, props));

    _this.registerInput = function (input) {
      return (0, _registerInput2.default)(_this, input);
    };

    _this.handleWindowResize = function () {
      var input = (0, _getInput2.default)(_this);
      var nextState = getStateFromProps(_this.props, _this.state.value, _this.initialWidth, input, _this.sizerEl);

      if (nextState.width !== _this.state.width) {
        _this.setState(nextState);
      }
    };

    _this.handleChange = function (e) {
      var value = e.target.value;

      var input = (0, _getInput2.default)(_this);
      var nextState = getStateFromProps(_this.props, value, _this.initialWidth, input, _this.sizerEl);

      if (nextState.width !== _this.state.width) {
        _this.setState(nextState);
      }

      if (_this.props.onChange) {
        _this.props.onChange(e);
      }
    };

    _this.state = getStateFromProps(props, props.value);
    return _this;
  }

  (0, _createClass3.default)(Autosize, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      if (typeof document === 'undefined') {
        return;
      }

      if (!sizersListEl) {
        sizersListEl = document.createElement('div');
        for (var key in sizerContainerStyle) {
          sizersListEl.style.setProperty(toDashCase(key), sizerContainerStyle[key]);
        }
        this.props.getSizerContainer().appendChild(sizersListEl);
      }

      this.sizerEl = document.createElement('span');
      sizersListEl.appendChild(this.sizerEl);

      window.addEventListener('resize', this.handleWindowResize);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (sizersListEl) {
        if (this.sizerEl) {
          sizersListEl.removeChild(this.sizerEl);
        }
        if (sizersListEl.childNodes.length === 0) {
          this.props.getSizerContainer().removeChild(sizersListEl);
          sizersListEl = null;
        }
      }

      this.sizerEl = null;

      window.removeEventListener('resize', this.handleWindowResize);
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (typeof window === 'undefined') {
        return;
      }

      var input = (0, _getInput2.default)(this);
      this.initialWidth = input.offsetWidth;
      var nextState = getStateFromProps(this.props, this.state.value, this.initialWidth, input, this.sizerEl);

      if (nextState.width !== this.state.width) {
        this.setState(nextState); // eslint-disable-line react/no-did-mount-set-state
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.value !== this.props.value || nextProps.padding !== this.props.padding) {
        var input = (0, _getInput2.default)(this);
        var nextState = getStateFromProps(nextProps, this.state.value, this.initialWidth, input, this.sizerEl);

        if (nextState.width !== this.state.width) {
          this.setState(nextState);
        }
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var children = _props.children;
      var style = _props.style;
      var placeholder = _props.placeholder;
      var value = _props.value;
      var width = this.state.width;

      var inputProps = {
        style: (0, _extends3.default)({}, style || {}, width ? { width: width } : {}),
        placeholder: placeholder,
        value: value,
        onChange: this.handleChange
      };

      return (0, _renderChild2.default)(children, inputProps, { width: width }, this.registerInput);
    }
  }]);
  return Autosize;
}(_react.PureComponent);

Autosize.defaultProps = {
  getSizerContainer: function getSizerContainer() {
    return document.body;
  },
  padding: 0
};
exports.default = Autosize;