'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _applyMaskToString = require('./utils/applyMaskToString');

var _applyMaskToString2 = _interopRequireDefault(_applyMaskToString);

var _getInput = require('./utils/getInput');

var _getInput2 = _interopRequireDefault(_getInput);

var _registerInput = require('./utils/registerInput');

var _registerInput2 = _interopRequireDefault(_registerInput);

var _renderChild = require('./utils/renderChild');

var _renderChild2 = _interopRequireDefault(_renderChild);

var _locateCursorPosition = require('./utils/locateCursorPosition');

var _locateCursorPosition2 = _interopRequireDefault(_locateCursorPosition);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getStateFromProps(props, value, lastIndex) {
  var stateValue = props.onValuePreUpdate(value);
  var preUpdateValue = stateValue;
  var processedValue = (0, _applyMaskToString2.default)(stateValue, props.pattern, props.emptyChar);
  var validatedValue = props.onValidate(stateValue, processedValue);

  if (validatedValue && validatedValue.result) {
    processedValue = validatedValue;
  } else if (validatedValue) {
    processedValue = {
      result: processedValue.result,
      unmaskedValue: processedValue.unmaskedValue,
      isValid: false
    };
  }

  if (processedValue.isValid) {
    stateValue = processedValue.result;
    lastIndex = processedValue.lastIndex;
  }

  if (!processedValue.unmaskedValue && props.placeholder) {
    stateValue = '';
  }

  return {
    value: stateValue,
    preUpdateValue: preUpdateValue,
    isValid: processedValue.isValid,
    unmaskedValue: processedValue.unmaskedValue,
    lastIndex: lastIndex
  };
}

var Mask = function (_PureComponent) {
  (0, _inherits3.default)(Mask, _PureComponent);

  function Mask(props) {
    (0, _classCallCheck3.default)(this, Mask);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Mask.__proto__ || (0, _getPrototypeOf2.default)(Mask)).call(this, props));

    _initialiseProps.call(_this);

    var value = props.value || '';
    _this.state = getStateFromProps(props, value, 0);
    return _this;
  }

  (0, _createClass3.default)(Mask, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (this.props.pattern !== nextProps.pattern || this.props.value !== nextProps.value || this.props.emptyChar !== nextProps.emptyChar) {
        var nextState = getStateFromProps(nextProps, nextProps.value || '', this.state.lastIndex);

        if (nextState.value !== this.state.value || nextState.lastIndex !== this.state.lastIndex) {
          this.setState(nextState);
        }
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      if (prevState.value !== this.state.value) {
        this.setSelectionRange(this.cursorPosition);
      }
    }
  }, {
    key: 'setSelectionRange',
    value: function setSelectionRange(position) {
      var input = (0, _getInput2.default)(this);
      if (input === document.activeElement) {
        input.setSelectionRange(position, position);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var children = _props.children;
      var placeholder = _props.placeholder;
      var value = this.state.value;

      var inputProps = {
        value: value,
        placeholder: placeholder,
        onInput: this.handleInput,
        onKeyDown: this.handleKeyDown,
        onMouseUp: this.handleMouseUp
      };

      return (0, _renderChild2.default)(children, inputProps, { value: value }, this.registerInput);
    }

    // works better for IE than onChange

  }]);
  return Mask;
}(_react.PureComponent);

Mask.defaultProps = {
  emptyChar: ' ',
  onValidate: function onValidate() {},
  onValuePreUpdate: function onValuePreUpdate(v) {
    return v;
  }
};

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.registerInput = function (input) {
    return (0, _registerInput2.default)(_this2, input);
  };

  this.handleInput = function (e) {
    var value = e.target.value;

    var nextState = getStateFromProps(_this2.props, value, _this2.state.lastIndex);
    if (!nextState.isValid) {
      e.preventDefault();
      setTimeout(function () {
        return _this2.setSelectionRange(_this2.cursorPosition);
      });
      return;
    }

    var cursorPosition = Math.min(nextState.lastIndex, (0, _locateCursorPosition2.default)(e.target.selectionStart, nextState.preUpdateValue, _this2.props.pattern, _this2.props.emptyChar));

    if (_this2.cursorPosition !== cursorPosition) {
      _this2.cursorPosition = cursorPosition;
      setTimeout(function () {
        return _this2.setSelectionRange(_this2.cursorPosition);
      });
    }

    if (e.target.value !== nextState.value) {
      e.target.value = nextState.value;
    }

    if (_this2.props.onUnmaskedValueChange) {
      _this2.props.onUnmaskedValueChange(nextState.unmaskedValue);
    }

    if (_this2.props.onChange) {
      _this2.props.onChange(e);
    }
  };

  this.handleKeyDown = function (e) {
    var input = (0, _getInput2.default)(_this2);

    if (e.key === 'ArrowRight' && input.selectionStart >= _this2.state.lastIndex) {
      _this2.setSelectionRange(_this2.state.lastIndex);
      e.preventDefault();
      return;
    }

    if (e.key === 'ArrowRight' && e.metaKey && input.selectionStart < _this2.state.lastIndex) {
      _this2.setSelectionRange(_this2.state.lastIndex);
      e.preventDefault();
      return;
    }

    if (_this2.props.onKeyDown) {
      _this2.props.onKeyDown(e);
    }
  };

  this.handleMouseUp = function (e) {
    var input = (0, _getInput2.default)(_this2);

    if (input.selectionStart >= _this2.state.lastIndex) {
      _this2.setSelectionRange(_this2.state.lastIndex);
    }

    if (_this2.props.onMouseUp) {
      _this2.props.onMouseUp(e);
    }
  };
};

exports.default = Mask;