'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _reactBase16Styling = require('react-base16-styling');

var _default = require('./themes/default');

var _default2 = _interopRequireDefault(_default);

var _inlineStylePrefixer = require('inline-style-prefixer');

var _inlineStylePrefixer2 = _interopRequireDefault(_inlineStylePrefixer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var prefixerInstance = void 0;

if (typeof window !== 'undefined' && window.navigator) {
  prefixerInstance = new _inlineStylePrefixer2.default(window.navigator);
} else {
  prefixerInstance = new _inlineStylePrefixer2.default({
    userAgent: 'Node.js (darwin; U; rv:v4.3.1) AppleWebKit/537.36 (KHTML, like Gecko)'
  });
}
var prefixer = prefixerInstance.prefix.bind(prefixerInstance);

function mixRGB(rgb1, rgb2) {
  var a = arguments.length <= 2 || arguments[2] === undefined ? 0.5 : arguments[2];

  var r = parseInt(rgb1.substr(1, 2), 16) * a + parseInt(rgb2.substr(1, 2), 16) * (1 - a);
  var g = parseInt(rgb1.substr(3, 2), 16) * a + parseInt(rgb2.substr(3, 2), 16) * (1 - a);
  var b = parseInt(rgb1.substr(5, 2), 16) * a + parseInt(rgb2.substr(5, 2), 16) * (1 - a);

  var rgb = ((Math.round(r) << 16) + (Math.round(g) << 8) + Math.round(b)).toString(16);
  return '#' + ('000000' + rgb.toString(16)).substr(-6);
}

var DAYPICKER_COLORS = {
  BACKGROUND: 'base00',
  NAVBAR_BACKGROUND: 'base01',
  NAVBAR_BORDER: 'base02',
  HOVER_BACKGROUND: 'base0D',
  WEEKDAY_TEXT: 'base03',
  TEXT: 'base05',
  DISABLED_TEXT: 'base03',
  DISABLED_BACKGROUND: 'base00',
  OUTSIDE_TEXT: 'base02',
  OUTSIDE_BACKGROUND: 'base00',
  TODAY_TEXT: 'base00',
  TODAY_BACKGROUND: 'base0C',
  SELECTED_TEXT: 'base00',
  SELECTED_BACKGROUND: 'base0D'
};

var DROPDOWN_COLORS = {
  TEXT: 'base05',
  BACKGROUND: 'base00',
  HEADER_TEXT: 'base05',
  HEADER_BACKGROUND: 'base01',
  HEADER_BORDER: 'base02',
  SELECTED_TEXT: 'base00',
  SELECTED_BACKGROUND: 'base0D',
  HOVER_BACKGROUND: 'base0D',
  DISABLED_TEXT: 'base03',
  DISABLED_BACKGROUND: 'base00',
  SEPARATOR: 'base01'
};

var prevSvg = function prevSvg(textColor) {
  return '\n  <svg width="26" height="50" viewBox="0 0 26 50" xmlns="http://www.w3.org/2000/svg">\n    <path\n      d="M25.893 49.124v-3.757L5.76 24.723 25.894 4.23V.476l-25 24.248 25 24.4z"\n      fill="' + textColor + '"\n      fill-rule="evenodd"\n    />\n  </svg>\n';
};

var nextSvg = function nextSvg(textColor) {
  return '\n  <svg width="26" height="50" viewBox="0 0 26 50" xmlns="http://www.w3.org/2000/svg">\n    <path\n      d="M.95 49.124v-3.757l20.133-20.644L.95 4.23V.476l25 24.248-25 24.4z"\n      fill="' + textColor + '"\n      fill-rule="evenodd"\n    />\n  </svg>\n';
};

var navButtonImg = function navButtonImg(type, textColor) {
  return {
    /* eslint-disable max-len */
    prev: 'data:image/svg+xml;base64,' + btoa(prevSvg(textColor)),
    next: 'data:image/svg+xml;base64,' + btoa(nextSvg(textColor))
  }[type];
};

var dayStyle = function dayStyle(base16Theme, mod, isHovered) {
  return {
    today: {
      boxShadow: '0px 0px 0 1px ' + base16Theme[DAYPICKER_COLORS.BACKGROUND] + ',\n      0px 0px 0 2px ' + base16Theme[DAYPICKER_COLORS.TODAY_BACKGROUND],
      fontWeight: '500'
    },
    disabled: {
      cursor: 'default',
      color: base16Theme[DAYPICKER_COLORS.DISABLED_TEXT],
      backgroundColor: base16Theme[DAYPICKER_COLORS.DISABLED_BACKGROUND]
    },
    outside: {
      cursor: 'default',
      color: base16Theme[DAYPICKER_COLORS.OUTSIDE_TEXT],
      backgroundColor: base16Theme[DAYPICKER_COLORS.OUTSIDE_BACKGROUND]
    },
    selected: {
      color: base16Theme[DAYPICKER_COLORS.SELECTED_TEXT],
      backgroundColor: base16Theme[DAYPICKER_COLORS.SELECTED_BACKGROUND]
    },
    default: {
      color: base16Theme[DAYPICKER_COLORS.TEXT],
      backgroundColor: isHovered ? mixRGB(base16Theme[DAYPICKER_COLORS.HOVER_BACKGROUND], base16Theme[DAYPICKER_COLORS.BACKGROUND], 0.1) : base16Theme[DAYPICKER_COLORS.BACKGROUND]
    }
  }[mod];
};

function getStylingFromBase16(base16Theme) {
  return {
    dayPicker: function dayPicker(_ref) {
      var style = _ref.style;
      return {
        style: (0, _extends3.default)({}, style, prefixer({
          display: 'flex',
          flexWrap: 'wrap',
          justifyContent: 'center',
          position: 'relative',
          userSelect: 'none',
          color: base16Theme[DAYPICKER_COLORS.TEXT],
          backgroundColor: base16Theme[DAYPICKER_COLORS.BACKGROUND]
        })),
        className: ''
      };
    },
    dayPickerMonth: function dayPickerMonth(_ref2) {
      var style = _ref2.style;
      return {
        style: (0, _extends3.default)({}, style, prefixer({
          userSelect: 'none',
          paddingBottom: '0.5rem'
        })),
        className: ''
      };
    },
    dayPickerNavBar: function dayPickerNavBar(_ref3) {
      var style = _ref3.style;
      return {
        style: (0, _extends3.default)({}, style, {
          position: 'absolute',
          left: '0',
          right: '0',
          padding: '1rem 0.5rem',
          backgroundColor: 'none'
        }),
        className: ''
      };
    },
    dayPickerNavButton: function dayPickerNavButton(_ref4, type) {
      var style = _ref4.style;
      return {
        style: (0, _extends3.default)({}, style, {
          left: type === 'prev' ? '1rem' : 'auto',
          right: type === 'next' ? '1rem' : 'auto',
          position: 'absolute',
          width: '1.5rem',
          height: '1.5rem',
          backgroundRepeat: 'no-repeat',
          backgroundPosition: 'center',
          backgroundSize: 'contain',
          cursor: 'pointer',
          backgroundImage: 'url("' + navButtonImg(type, base16Theme[DAYPICKER_COLORS.TEXT]) + '")'
        }),
        className: ''
      };
    },
    dayPickerCaption: function dayPickerCaption(_ref5) {
      var style = _ref5.style;
      return {
        style: (0, _extends3.default)({}, style, {
          height: '3.5rem',
          padding: '0.5rem',
          backgroundColor: base16Theme[DAYPICKER_COLORS.NAVBAR_BACKGROUND],
          borderBottom: '1px solid ' + base16Theme[DAYPICKER_COLORS.NAVBAR_BORDER],
          textAlign: 'center'
        }),
        className: ''
      };
    },
    dayPickerWeekdays: function dayPickerWeekdays(_ref6) {
      var style = _ref6.style;
      return {
        style: (0, _extends3.default)({}, style, {
          display: 'flex',
          padding: '0 0.5rem'
        }),
        className: ''
      };
    },
    dayPickerWeekdaysRow: function dayPickerWeekdaysRow(_ref7) {
      var style = _ref7.style;
      return {
        style: (0, _extends3.default)({}, style, prefixer({
          display: 'flex'
        })),
        className: ''
      };
    },
    dayPickerWeekday: function dayPickerWeekday(_ref8) {
      var style = _ref8.style;
      return {
        style: (0, _extends3.default)({}, style, {
          width: '3.2rem',
          margin: '0.1rem',
          padding: '.5rem',
          fontSize: '.875em',
          textAlign: 'center'
        }),
        className: ''
      };
    },
    dayPickerMonthWrapper: function dayPickerMonthWrapper(_ref9) {
      var style = _ref9.style;
      return {
        style: style,
        className: ''
      };
    },
    dayPickerWeek: function dayPickerWeek(_ref10) {
      var style = _ref10.style;
      return {
        style: (0, _extends3.default)({}, style, prefixer({
          display: 'flex',
          padding: '0 0.5rem'
        })),
        className: ''
      };
    },
    dayPickerDay: function dayPickerDay(_ref11, day, modifiers, isHovered) {
      var style = _ref11.style;
      var className = _ref11.className;
      return {
        style: (0, _extends3.default)({}, style, {
          outline: 'none'
        }, dayStyle(base16Theme, 'default', isHovered), {
          padding: '0.5rem',
          borderRadius: '10rem',
          width: '3.2rem',
          height: '3.2rem',
          margin: '0.15rem',
          textAlign: 'center',
          cursor: 'pointer',
          verticalAlign: 'middle'
        }, modifiers.reduce(function (s, mod) {
          return (0, _extends3.default)({}, s, dayStyle(base16Theme, mod));
        }, {})),
        className: ''
      };
    },

    inputEnhancementsListHeader: prefixer({
      flex: '0 0 auto',
      minHeight: '3rem',
      fontSize: '0.8em',
      color: base16Theme[DROPDOWN_COLORS.HEADER_TEXT],
      backgroundColor: base16Theme[DROPDOWN_COLORS.HEADER_BACKGROUND],
      padding: '0.5rem 1rem',
      borderBottom: '1px solid ' + base16Theme[DROPDOWN_COLORS.HEADER_BORDER]
    }),
    inputEnhancementsListOptions: prefixer({
      flex: '1 1 auto',
      overflowY: 'auto',
      // fix for IE
      // https://connect.microsoft.com/IE/feedback/details/802625
      maxHeight: '27rem'
    }),
    inputEnhancementsOption: function inputEnhancementsOption(_ref12, highlighted, disabled, hovered) {
      var style = _ref12.style;
      return {
        style: (0, _extends3.default)({}, style, disabled ? {
          padding: '1rem 1.5rem',
          cursor: 'pointer',
          color: base16Theme[DROPDOWN_COLORS.DISABLED_TEXT]
        } : highlighted ? {
          padding: '1rem 1.5rem',
          cursor: 'pointer',
          color: base16Theme[DROPDOWN_COLORS.SELECTED_TEXT],
          backgroundColor: base16Theme[DROPDOWN_COLORS.SELECTED_BACKGROUND]
        } : {
          padding: '1rem 1.5rem',
          cursor: 'pointer',
          color: base16Theme[DROPDOWN_COLORS.TEXT],
          backgroundColor: hovered ? mixRGB(base16Theme[DROPDOWN_COLORS.HOVER_BACKGROUND], base16Theme[DROPDOWN_COLORS.BACKGROUND], 0.1) : base16Theme[DROPDOWN_COLORS.BACKGROUND]
        })
      };
    },
    inputEnhancementsSeparator: {
      margin: '0.5rem 0',
      width: '100%',
      height: '1px',
      borderTop: '1px solid ' + base16Theme[DROPDOWN_COLORS.SEPARATOR]
    },

    inputEnhancementsPopupWrapper: {
      position: 'relative',
      display: 'inline-block'
    },
    inputEnhancementsCaret: {
      position: 'absolute',
      right: '5px',
      top: 0,
      paddingTop: '5px',
      verticalAlign: 'middle',
      paddingLeft: '3px',
      width: '10px'
    },
    inputEnhancementsCaretSvg: function inputEnhancementsCaretSvg(_ref13, isActive, hovered) {
      var style = _ref13.style;
      return {
        style: (0, _extends3.default)({}, style, prefixer({
          display: 'inline-block',
          opacity: hovered || isActive ? 1 : 0,
          transition: 'opacity 0.15s linear, transform 0.15s linear',
          transform: hovered || isActive ? 'translateY(0)' : 'translateY(5px)'
        }))
      };
    },
    inputEnhancementsPopup: prefixer({
      display: 'flex',
      position: 'absolute',
      left: 0,
      top: '100%',
      zIndex: 10000,
      maxHeight: '30rem',
      minWidth: '22rem',
      backgroundColor: base16Theme[DROPDOWN_COLORS.BACKGROUND],
      boxShadow: '1px 1px 4px rgba(100, 100, 100, 0.3)',
      flexDirection: 'column'
    }),
    inputEnhancementsInput: {
      paddingRight: '15px'
    }
  };
}

exports.default = (0, _reactBase16Styling.createStyling)(getStylingFromBase16, {
  defaultBase16: _default2.default
});